/* global DeviceManager, LibraryManager, utils */

(function () {
  "use strict";

  var isDebugOn = true;

  var triggerTypeEnum = {webhook: "webhook", touch: "touch"};


  function ActionEvent(domElemSource, id, triggerType, triggerCode, actionType, layoutCode, source, sourceId, target, targetId, widgetId, layoutId, commandCode) {
    this.domElemSource = domElemSource;
    this.id = id;
    this.triggerType = triggerType;
    this.triggerCode = triggerCode;
    this.actionType = actionType;
    this.layoutCode = layoutCode;
    this.source = source;
    this.sourceId = sourceId;
    this.target = target;
    this.targetId = targetId;
    this.widgetId = widgetId;
    this.layoutId = layoutId;
    this.commandCode = commandCode;

  };

  ActionEvent.prototype.equals = function (domElemSource, id, triggerType, triggerCode, actionType, layoutCode, source, sourceId, target, targetId, widgetId, commandCode) {
    return this.domElemSource == domElemSource && this.id == id
      && this.triggerType == triggerType && this.triggerCode == triggerCode
      && this.actionType == actionType && this.layoutCode == layoutCode && this.source == source
      && this.sourceId == sourceId && this.target == target && this.targetId == targetId
      && this.widgetId == widgetId && this.commandCode == commandCode;
    ;
  };

  window.InteractiveManager = new function () {
    // <editor-fold defaultstate="collapsed" desc="Private variables">
    // Load the Service and call the local endpoint
    var port = localStorage.internalwsPort || 9696;

    var v_registeredActions = [];

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Private functions">
//    var int_webhookcb = function(data){
//      console.log('InteractiveManager webhook data',data);
//    };
//
//    ServiceProxy.ws.register(int_webhookcb);

    // <editor-fold defaultstate="collapsed" desc="IPC">
    var v_iows = io('http://127.0.0.1:' + port);

    v_iows.on('info', function (fn) {
      var dt = {
        "hardwareKey": "",
        "displayName": "",
        "timeZone": "",
        "latitude": null,
        "longitude": null
      };

      dt.hardwareKey = DeviceManager.Config.getHardwareKey();
      dt.displayName = DeviceManager.Config.getDisplayName();
      dt.timeZone = DeviceManager.Device.getTimeZone();

      fn(dt);
    });

    v_iows.on('trigger', function (data, fn) {
      console.info("internalws trigger", v_registeredActions);
      if (data.trigger) {
        var actid = findEventIndexTriggerCode(data.trigger);
        if (actid > -1) {
          m_handleEvent(v_registeredActions[actid]);
          fn("Triggered");
          return;
        }
        fn("No action with code " + data.trigger);
        return;
      }
      fn("No trigger data");
    });

    v_iows.on('/duration/expire', function (data, fn) {
      console.info("internalws /duration/expire");
      fn();
    });

    v_iows.on('/duration/extend', function (data, fn) {
      console.info("internalws /duration/extend");
      fn();
    });

    v_iows.on('/duration/set', function (data, fn) {
      console.info("internalws /duration/set");
      fn();
    });

    v_iows.on('/criteria', function(data, fn) {
      //console.info("internalws /criteria", data);
      if (data && data.constructor && (data.constructor == Array)){
        for (let i = 0; i < data.length; i++) {
          var o = data[i];
          ScheduleManager.Criteria.set(o.metric, o.value, o.ttl);
        }
      }

      fn();
    });
    // </editor-fold>


    function findEventIndex(id) {
      for (var i = 0; i < v_registeredActions.length; i++) {
        if (v_registeredActions[i].id == id) {
          return i;
        }
      }
      return -1;
    }

    function findEventIndexByDomElement(elem) {
      for (var i = 0; i < v_registeredActions.length; i++) {
        if (v_registeredActions[i].domElemSource == elem) {
          return i;
        }
      }
      return -1;
    }

    function findEventIndexTriggerCode(code) {
      for (var i = 0; i < v_registeredActions.length; i++) {
        if (v_registeredActions[i].triggerCode == code) {
          return i;
        }
      }
      return -1;
    }


    function m_handleEvent(actevt) {
      console.info(actevt);
      switch (actevt.actionType) {
        case 'next':
          if (actevt.target === "region") {
            LayoutManager.nextWidget(actevt.targetId);
          } else if (actevt.target === "layout") {
            console.warn("Action next layout isn't wired up");
          } else if (actevt.target === "screen") {
            LayoutManager.nextLayout();
          }
          break;
        case 'previous':
          if (actevt.target === "region") {
            LayoutManager.previousWidget(actevt.targetId);
          } else if (actevt.target === "layout") {
            console.warn("Action previous layout isn't wired up");
          } else if (actevt.target === "screen") {
            LayoutManager.previousLayout();
          }
          break;
        case 'navLayout':
          var l = LibraryManager.getFileByActionCode(actevt.layoutCode);
          if (l) {
            LayoutManager.setLayoutByAction(l.id, actevt.triggerType);
          } else {
            console.warn('Layout with code ' + actevt.layoutCode + " could not be found!");
          }


//          var l = LayoutManager.getLayoutByCode(actevt.layoutCode);
//          if (l){
//            LayoutManager.setLayoutByAction(l.id,actevt.triggerType);
//          }else{
//            console.warn('Layout with code ' + actevt.layoutCode + " could not be found!");
//          }
          break;
        case 'navWidget':
          if (actevt.target === "screen") {
            LayoutManager.playWidgetToRegion(actevt.widgetId, "screen");
          } else {
            LayoutManager.playWidgetToRegion(actevt.widgetId, actevt.targetId);
          }
          break;
        case 'command':
          if (actevt.target === "screen") {
            ScheduleManager.Commands.commandAction(actevt.commandCode);
          } else {
            Logger.cmsFault(undefined, undefined,
              {
                code: ERRORS.FaultCodes.GeneralError,
                reason: "Command actions must be targeted to the screen",
                method: "handleEvent",
                thread: "InteractiveManager",
                date: new Date(),
                expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
              }
            );
          }
          break;
        default:
          console.warn('Action type unknown', actevt.actionType);

          break;
      }
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Public functions">
    /**
     * Add a new event action.
     * @param {type} domElemSource
     * @param {type} id
     * @param {type} triggerType
     * @param {type} triggerCode
     * @param {type} actionType
     * @param {type} layoutCode
     * @param {type} source
     * @param {type} sourceId
     * @param {type} target
     * @param {type} targetId
     * @param {type} widgetId
     * @returns {Boolean} True if added, false otherwise.
     */
    this.addEventAction = function (domElemSource, id, triggerType, triggerCode, actionType, layoutCode, source, sourceId, target, targetId, widgetId, layoutId, commandCode) {
      var idx = findEventIndex(id);
      if (triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
        domElemSource.addEventListener('click', InteractiveManager.handleTouchEvent);
      }
      if (idx === -1) {
        if (isDebugOn) {
          console.info('Interactive.addEventAction [new]:', domElemSource, id);
        }
        var n = new ActionEvent(domElemSource, id, triggerType, triggerCode, actionType, layoutCode, source, sourceId, target, targetId, widgetId, layoutId, commandCode);
        v_registeredActions.push(n);
        return true;
      }
      if (isDebugOn) {
        console.info('Interactive.addEventAction [update]:', domElemSource, id);
      }
      v_registeredActions[idx].domElemSource = domElemSource;
      return false;
    };

    /**
     * Remove an event action by it's id
     * @param {type} eventId
     */
    this.removeEventAction = function (eventId) {
      var i = findEventIndex(eventId);
      if (isDebugOn) {
        console.info('Interactive.removeEventAction:', i, eventId);
      }
      if (i >= 0) {
        if (v_registeredActions[i].triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
          v_registeredActions[i].domElemSource.removeEventListener('click', InteractiveManager.handleTouchEvent);
        }
        v_registeredActions.splice(i, 1);
      }
    };

    /**
     * Remove all event actions.
     * @param {String} layoutId
     * @returns {undefined}
     */
    this.removeLayoutEventActions = function (layoutId) {
      var actLength = v_registeredActions.length;
      for (var i = actLength - 1; i >= 0; --i) {
        if (v_registeredActions[i].layoutId == layoutId) {
          v_registeredActions.splice(i, 1);
        }
      }
    };

    this.handleTouchEvent = function (evt) {
      var actid = findEventIndexByDomElement(evt.currentTarget);
      if (isDebugOn) {
        console.info('Interactive.handleTouchEvent:', actid);
      }
      if (actid > -1) {
        m_handleEvent(v_registeredActions[actid]);
      }
    };

    this.handleWebHookEvent = function (code) {
      var actid = findEventIndexTriggerCode(code);
      if (isDebugOn) {
        console.info('Interactive.handleWebHookEvent:', actid);
      }
      if (actid > -1) {
        m_handleEvent(v_registeredActions[actid]);
      }
    };

    /**
     * Get all actions added by schedule.
     * @returns {Actions[]} An array of actions
     */
    this.getScheduledActions = function () {
      var scheduledActions = [];
      for (var i = 0, max = v_registeredActions.length; i < max; i++) {
        if (v_registeredActions[i].source === "schedule") {
          scheduledActions.push(v_registeredActions[i]);
        }
      }
      return scheduledActions;
    };

    // <editor-fold desc="Data Collector functions">

    /**
     * Inter-process communication
     */
    this.ipc = new function _ipc() {
      var ipcInstance = this;

      /**
       * Handles the DataConnectors
       */
      ipcInstance.dataConnectors = new function() {
        // eslint-disable-next-line no-invalid-this
        var dataConnectorsInstance = this;

        // Dome element where data connectors live
        var domContainer;

        // List of data connectors
        var dataConList = {};

        /**
         * Add a data connector
         * @param {string} key Key of the data connector
         * @param {string} userJSCode The file that has the JavaScript code
         * @param {string} dataParams The parameters for the data connector
         */
        dataConnectorsInstance.add = function(key, userJSCode, dataParams) {
          console.log('DataConnectors: Add');

          var userScript = document.createElement('script');

          var localwsport = localStorage.internalwsPort || "9696";
          userScript.setAttribute('src', 'http://127.0.0.1:' + localwsport + '/content/' + userJSCode + '');
          userScript.setAttribute('onload', 'onInit()');

          // Parameter parsing
          var params = dataParams.split('&');
          // Always add the dataSetId to the data connector widget
          params.push('dataSetId='+key);

          var eIFrm = document.createElement('iFrame');
          eIFrm.id = key;
          eIFrm.addEventListener('load', function(evt) {
            console.log('DataConenctors: iFrame loaded', evt);
            var pair = null;
            params.forEach(function(d) {
              pair = d.split('=');
              eIFrm.contentWindow[pair[0]] = pair[1];
            });

            eIFrm.contentDocument.head.appendChild(userScript);
          });

          eIFrm.setAttribute('src', 'http://127.0.0.1:' + localwsport + '/libs/dataCollector.html');

          dataConList[key] = {
            userJSCode: userJSCode,
            dataParams: dataParams,
            iFrame: eIFrm
          };
          domContainer.appendChild(eIFrm);
        };

        /**
         * Remove a data connector
         * @param {string} key Key of the data connector
         * @return {boolean} True if remove it else false
         */
        dataConnectorsInstance.remove = function(key) {
          if (dataConList[key]) {
            dataConList[key].iFrame.remove();
            delete dataConList[key];
            return true;
          }
          return false;
        };

        /**
         * Get a data connector
         * @param {string} key Key of the data connector
         * @return {object} Return an object with user code,
         * parameters and the iFrame that lives
         */
        dataConnectorsInstance.get = function(key) {
          return dataConList[key];
        };

        dataConnectorsInstance.keys = function() {
          return Object.keys(dataConList);
        };

        document.addEventListener('DOMContentLoaded', function (){
          domContainer = document.getElementById('dataConnectorContainers');
        });
      };

      window.addEventListener('message', function(evt) {
        // console.info('IPC:', evt);
        if (evt.origin.startsWith('http://127.0.0.1') || evt.origin.startsWith('file://')) {
          try {
            var data;
            if (typeof(evt.data)==='string') {
              data = JSON.parse(evt.data);
            } else if (typeof(evt.data)==='object') {
              data = evt.data;
            } else {
              console.warn('IPC: unknown data format');
              return;
            }
            if (data.ctrl) {
              if (data.ctrl === 'rtNotifyHost') {
                var ifms = document.getElementsByTagName('iframe');
                [].slice.call(ifms).forEach(function(ifm) {
                  // TODO: Think about filtering based on the datasetId and/or widgetId
                  ifm.contentWindow.postMessage({
                    ctrl: 'rtNotifyData',
                    data: data.data
                  }, '*');
                });
              } else if (data.ctrl === 'rtSetCriteria') {
                ScheduleManager.Criteria.set(
                  data.data.metric,
                  data.data.value,
                  data.data.TTL
                );
              }
            }
          } catch (e) {
            console.error(e);
          }
        }
      });
    };

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Test functions">
    this.setDebug = function (state) {
      isDebugOn = state === true;
      return isDebugOn;
    };

    this.showActions = function () {
      console.log('Actions', v_registeredActions);
    };
    // </editor-fold>
    // </editor-fold>
  };

  utils.const(window.InteractiveManager, 'TRIGGERTYPE_WEBHOOK', triggerTypeEnum.webhook);
  utils.const(window.InteractiveManager, 'TRIGGERTYPE_TOUCH', triggerTypeEnum.touch);
}());

