var express = require('express');
var http = require('http');
var https = require('https');
var path = require('path');
var fs = require('fs');
var URL = require('url');

var app = express();

var contentPath = '/opt/usr/home/owner/content/Downloads/';

var localHostOnlyAccessFlag = false;

/* eslint-disable max-len */
/**
 * Middleware to check if access is from localhost
 * @param {Request<ParamsDictionary, any, any, ParsedQs, Record<string, any>>} req The request
 * @param {Response<any, Record<string, any>>} res The response
 * @param {function} next Next in sequence
 */
function localHostOnly(req, res, next) {
  if (!(req.hostname === '127.0.0.1' || req.hostname === 'localhost')) {
    res.statusCode = 403;
    res.send();
  } else {
    next();
  }
}
/* eslint-enable max-len */


function userDefinedLocalHostOnly(req, res, next) {
  if (localHostOnlyAccessFlag){
    localHostOnly(req, res, next);
  } else {
    next();
  }
}

app.disable('x-powered-by');
// Middle-ware
app.use(function(req, res, next) {
  res.header('Access-Control-Allow-Origin', '*');
  res.header(
    'Access-Control-Allow-Methods',
    'GET, POST, PUT, DELETE, OPTIONS');
  res.header(
    'Access-Control-Allow-Headers',
    'Origin, X-Requested-With, Content-Type, Accept');
  next();
});


// app.use(express.json());
app.use( express.raw({limit: '50mb'}) );
app.use( express.text({limit: '50mb'}) );
app.use( express.json({limit: '50mb'}) );
app.use(express.urlencoded({
  limit: '50mb',
  extended: true,
  parameterLimit: 50000
}));

// app.use('/content',express.static(contentPath));

// Don't use the express.static middleware. Do it by hand
// app.use('/libs',express.static(path.join(__dirname, './libs')));
var libDir = path.join(__dirname, 'libs/');

var server = http.createServer(app);
var io = require('socket.io')(server);

var vClient = null;

io.addListener('connection', function(client) {
  vClient = client;
//  client.emit('log', JSON.stringify({data:'ws hello'}));
});

// /**/
// app.get('/l', function(req, res) {
//   var p = __dirname+ '/'+req.body.path;
//   try {
//     fs.readdir(p, (err, files) => {
//       res.send({
//         path: p,
//         files: files
//       });
//     });
//   } catch (e) {
//     res.send({error: e.code, message: e.message});
//   }
// });
// /**/
//
// app.get('/cont', function(req, res) {
//   // var p = __dirname+ '/'+req.body.path;
//   try {
//     var fobj = {};
//     var filesDetails = [];
//     fs.readdir(contentPath, (err, files) => {
//       for (var i = 0, max = files.length; i < max; i++) {
//         fobj = {filename: files[i]};
//         fobj.details = fs.statSync(contentPath+files[i]);
//         filesDetails.push(fobj);
//       }
//       res.send({
//         path: contentPath,
//         files: filesDetails
//       });
//     });
//   } catch (e) {
//     res.send({error: e.code, message: e.message});
//   }
// });

app.get('/status', userDefinedLocalHostOnly, function(req, res) {
  res.send({
    status: 'OK'
  });
});
/*
    env: process.env,
    versions: process.versions
*/
app.post('/info', userDefinedLocalHostOnly, function(req, res) {
  if (vClient) {
    vClient.emit('info', function(data) {
      res.send(data);
      //        {
      //        "hardwareKey": "hwKey",
      //        "displayName": "displayName",
      //        "timeZone": "Europe/Athens",
      //        "latitude": null,
      //        "longitude": null
      //      }
    });
  } else {
    res.status(503).send();
  }
});

app.post('/trigger', userDefinedLocalHostOnly, function(req, res) {
  if (vClient) {
    vClient.emit('trigger', req.body, function(data) {
      res.send({status: 'OK', data: data});
    });
  } else {
    res.status(503).send();
  }
});

app.post('/duration/expire', userDefinedLocalHostOnly, function(req, res) {
  if (vClient) {
    vClient.emit('/duration/expire', req.body, function(data) {
      res.send({status: 'OK', data: data});
    });
  } else {
    res.status(503).send();
  }
});

app.post('/duration/extend', userDefinedLocalHostOnly, function(req, res) {
  if (vClient) {
    vClient.emit('/duration/extend', req.body, function(data) {
      res.send({status: 'OK', data: data});
    });
  } else {
    res.status(503).send();
  }
});

app.post('/duration/set', userDefinedLocalHostOnly, function(req, res) {
  if (vClient) {
    vClient.emit('/duration/set', req.body, function(data) {
      res.send({status: 'OK', data: data});
    });
  } else {
    res.status(503).send();
  }
});

// // Make the needed files available as per express example
// // /files/* is accessed via req.params[0]
// // but here we name it :file
// app.get('/libs/:file(*)', function(req, res, next){
//   res.download(req.params.file, { root: libDir }, function (err) {
//     if (!err) return; // file sent
//     if (err.status !== 404) return next(err); // non-404 error
//     // file for download not found
//     res.statusCode = 404;
//     res.send('Not found');
//   });
// });

app.get('/content/:file(*)', localHostOnly, function(req, res, next) {
  var options = {
    root: contentPath,
    dotfiles: 'deny',
    headers: {
      'x-timestamp': Date.now(),
      'x-sent': true
    }
  };

  res.sendFile(req.params.file, options, function(err) {
    if (!err) return; // file sent
    if (err.status !== 404) return next(err); // non-404 error
    // file for download not found
    res.statusCode = 404;
    res.send('Not found');
  });
});

// Files that are for the use of the player
// but does not belong to the internal media library
app.get('/libs/:file(*)', localHostOnly, function(req, res, next) {
  var options = {
    root: libDir,
    dotfiles: 'deny',
    headers: {
      'x-timestamp': Date.now(),
      'x-sent': true
    }
  };

  res.sendFile(req.params.file, options, function(err) {
    if (!err) return; // file sent
    if (err.status !== 404) return next(err); // non-404 error
    // file for download not found
    res.statusCode = 404;
    res.send('Not found');
  });
});

// RealTime data handling
var rtCache = {};
app.get('/realtime', userDefinedLocalHostOnly, function(req, res) {
  var dataKey = req.query.dataKey;
  if (!dataKey) {
    res.status(400).send({
      status: 'ERROR',
      message: 'Paramenter not set'
    });
    return;
  }
  if (rtCache[dataKey]) {
    res.send(rtCache[dataKey]);
  } else {
    if (fs.existsSync(contentPath+dataKey+'.json')) {
      res.sendFile(dataKey+'.json', {root: contentPath}, function(err) {
        res.end();
        // if (err) throw(err);
      });
    } else {
      res.status(404).send();
    }
  }
});

app.post('/realtime', userDefinedLocalHostOnly, function(req, res) {
  var dataKey = req.query.dataKey;
  if (!dataKey) {
    res.status(400).send({status: 'ERROR', message: 'Paramenter not set'});
    return;
  }
  rtCache[dataKey] = req.body;
  res.status(200).send();
});

app.post('/realtime/cache', localHostOnly, function(req, res) {
  res.send(rtCache);
});


app.post('/repost', localHostOnly, (req, res) => {
  try {
    var method = req.body.type || 'GET';
    var headers = req.body.headers;
    var body = req.body.data;
    var apiUrl = req.body.path;
    var url = URL.parse(apiUrl);

    var options = {
      method: method,
      hostname: url.hostname,
      port: url.port || (url.protocol === 'https:' ? 443 : 80),
      path: url.pathname,
      headers: headers,
      rejectUnauthorized: false
    };

    var httpModule;
    if (url.protocol === 'https:') {
      httpModule = https;
    } else {
      httpModule = http;
    }
    // var httpModule = (url.protocol === 'https:' ? https : http);

    var proxyReq = httpModule.request(options, (proxyRes) => {
      var responseData = '';
      proxyRes.on('data', (chunk) => {
        responseData += chunk;
      });
      proxyRes.on('end', () => {
        res.setHeader('Content-Type', 'application/json');
        res.send({status: proxyRes.statusCode, data: responseData});
      });
    });

    if (method === 'POST' || method === 'PUT') {
      proxyReq.write(JSON.stringify(body));
    }

    proxyReq.on('error', (e) => {
      console.log(`problem with request: ${e.message}`);
      res.status(500).json({error: e.message});
    });

    proxyReq.end();
  } catch (error) {
    console.error(error, error.stack);
    res.status(500).json({error: error.message});
  }
});

app.post('/criteria', userDefinedLocalHostOnly, (req, res) => {
  try {
    if (vClient) {
      var reqData = req.body;
      vClient.emit('/criteria', reqData,
        function(data) {
          res.send(data);
        }
      );
    } else {
      res.status(503).send();
    }
  } catch (e) {
    res.status(500).json({error: error.message});
    console.log(e);
  }
});

app.use(function(req, res, next) {
  res.connection.destroy();
});


module.exports = {
  server: server,
  app: app,
  io: io,
  localHostAccessOnly: localHostOnlyAccessFlag
};

if (require.main === module) {
  console.log('Server started');
  server.listen(9696);
}
